//
//  NSArray+HigherOrderFunctions.m
//  SubjectiveKit
//
//  Created by Mizushima Kota on 12/05/02.
//  Copyright (c) 2012 onion_lang.org. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "NSArray+HigherOrderFunctions.h"

@implementation NSArray (HigherOrderFunctions)

- (void)eachWithIndex:(Fn2)fn {
	for (NSInteger i = 0; i < self.count; i++) {
		fn([self objectAtIndex:i], [NSNumber numberWithInteger:i]);
	}
	
}

- (NSArray*)map:(Fn1)fn {
	NSMutableArray* newArray = [NSMutableArray arrayWithCapacity:self.count];
	for (id e in self) {
		[newArray addObject:fn(e)];
	}
	return newArray;
}

- (NSArray*)bind:(Fn1)fn {
    NSMutableArray* newArray = [NSMutableArray array];
    for (id e in self) {
        [newArray addObjectsFromArray:(NSArray*)fn(e)];
    }
    return newArray;
}

- (NSArray*)takeWhile:(Pred1)pred {
    NSMutableArray* newArray = [NSMutableArray array];
    for (id e in self) {
        if (pred(e)) [newArray addObject:e];
    }
    return newArray;
}

- (NSArray*)dropWhile:(Pred1)pred {
    NSMutableArray* newArray = [NSMutableArray array];
    NSInteger i;
    for (i = 0; i < self.count; i++) if(!pred(self[i])) break;
    for (; i < self.count; i++) [newArray addObject:self[i]];
    return newArray;
}

- (NSArray*)filter:(Pred1)pred {
	NSMutableArray* newArray = [NSMutableArray array];
	for (id e in self) {
		if (pred(e)) [newArray addObject:e];
	}
	return newArray;
}

- (NSArray*) filterNot:(Pred1)pred {
	return [self filter:^(id e) { return (BOOL)(!pred(e)); }];
}

- (id)foldLeft:(id)seed of:(Fn2)fun {
	id result = seed;
	for (id e in self) result = fun(result, e);
	return result;
}

- (id)foldRight:(id)seed of:(Fn2)fun {
	id result = seed;
	for (NSInteger i = [self count] - 1; 0 <= i; i--) {
		id e = [self objectAtIndex:i];
		result = fun(e, result);
	}
	return result;
}

- (NSString*)join:(NSString*)separator {
	NSInteger count = self.count;
	if (count == 0) return @"";
	
	NSEnumerator* enumerator = [self objectEnumerator];
	NSMutableString* result = [NSMutableString string];
	
	NSString* firstString = [[enumerator nextObject] description];
	[result appendString:firstString];
	for (id e in enumerator) {
		[result appendString:separator];
		[result appendString:[e description]];
	}
	
	return result;
}

- (NSSet*)toSet {
    return [NSSet setWithArray:self];
}

- (NSObject*)sum:(NSObject<Monoid>*)concept {
    return [self foldLeft:[concept zero] of:^(NSObject* a, NSObject* b) { return [concept plus:a by:b]; }];
}


@end
