//
//  NSSet+HigherOrderFunctions.h
//  SubjectiveKit
//
//  Created by Kota Mizushima on 2012/10/19.
//
//

#import <Foundation/Foundation.h>
#import "FunctionTypes.h"
#import "Monoid.h"

@interface NSSet (HigherOrderFunctions)

/**
 * For each element , call fn(element) sequentially.
 */
- (void)each:(Fn1)fn;

/**
 * Returns a new |NSSet| generated from self.
 * The each element of the |NSSet| is the result of
 * applying |fn(element)|.
 */
- (NSSet*)map:(Fn1)fn;

/**
 * Returns a new |NSSet| excluding the element
 * which doesn't satisfy |pred(e)|.
 */
- (NSSet*)filter:(Pred1)pred;

/**
 * Returns a new |NSSet| excluding the element
 * which satisfy |pred(e)|.  This method is inverse
 * of |filter|.
 */
- (NSSet*)filterNot:(Pred1)pred;

/**
 * Returns an object.  The result object is
 * |fun(... fun(fun(seed, e1), e2), ...en)|, which
 * |n| is |self.count|.  The order of function application
 * is not deterministic.
 */
- (id)fold:(id)seed of:(Fn2)fun;

/**
 * Returns a |NSString| by concatenating each element
 * of |self| separated by |separator|.
 */
- (NSString*)join:(NSString*)separator;

/**
 * Sum of all element using |concept|, which is a |NSObject<Monoid>*|.
 * element type should be equal to concept type.  Since |NSSet| is
 * not ordered, |concept#plus:| should satisfy commutative law.
 */
- (NSObject*)sum:(NSObject<Monoid>*)concept;


@end
