//
//  NSSet+HigherOrderFunctions.m
//  SubjectiveKit
//
//  Created by Kota Mizushima on 2012/10/19.
//
//

#import "NSSet+HigherOrderFunctions.h"

@implementation NSSet (HigherOrderFunctions)

- (void)each:(Fn1)fn {
    for (id e in self) fn(e);
}

- (NSSet*)map:(Fn1)fn {
    NSMutableSet* newSet = [NSMutableSet setWithCapacity:self.count];
    for (id e in self) {
        [newSet addObject:fn(e)];
    }
    return newSet;
}

- (NSSet*)filter:(Pred1)pred {
    NSMutableSet* newSet = [NSMutableSet setWithCapacity:self.count];
    for (id e in self) {
        if (pred(e)) [newSet addObject:e];
    }
    return newSet;
}

- (NSSet*)filterNot:(Pred1)pred {
    return [self filter:^BOOL(id e) { return !pred(e); }];
}

- (id)fold:(id)seed of:(Fn2)fun {
    id result = seed;
    for (id e in self) {
        result = fun(result, e);
    }
    return result;
}

- (NSString*)join:(NSString*)separator {
    NSMutableString* result = [NSMutableString string];
    if (self.count == 0) return @"";
    NSEnumerator* enumerator = self.objectEnumerator;
    [result appendFormat:@"%@", enumerator.nextObject];
    for (id e in enumerator) [result appendFormat:@"%@%@", separator, e];
    return result;
}


- (NSArray*)toArray {
    NSMutableArray* array = [NSMutableArray array];
    for (id e in self) [array addObject:e];
    return array;
}

- (NSObject*)sum:(NSObject<Monoid>*)concept {
    return [self fold:[concept zero] of:^(NSObject* a, NSObject* b) { return [concept plus:a by:b]; }];
}


@end
