//
//  SubjectiveKitTests.m
//  SubjectiveKitTests
//
//  Created by Mizushima Kota on 12/05/02.
//  Copyright (c) 2012年 __MyCompanyName__. All rights reserved.
//

#import "NSArrayHigherOrderFunctionsTestCase.h"
#import "NSArray+HigherOrderFunctions.h"
#import "FunctionTypes.h"
#import "NSMutableArray+HigherOrderFunctions.h"
#import "Monoids.h"

@implementation NSArrayHigherOrderFunctionsTestCase

- (NSNumber*)number:(NSInteger)n {
	return [NSNumber numberWithInteger:n];
}


- (void)setUp
{
    [super setUp];
    
    // Set-up code here.
}

- (void)tearDown
{
    // Tear-down code here.
    
    [super tearDown];
}

- (void)testMap {
	NSArray *array = [NSArray arrayWithObjects:@"A", @"B", @"C", nil];
	
	NSArray *newArray = [array map:^(NSString* e){ return [e stringByAppendingString:@"X"]; }];
	
	NSArray *expected = [NSArray arrayWithObjects:@"AX", @"BX", @"CX", nil];
	
	STAssertEqualObjects(newArray, expected, nil);
}

- (void)testBind {
	NSArray *ar1 = @[@1, @2];
    NSArray *ar2 = @[@1, @2];
    
    NSArray *newArray = [ar1 bind:^(NSNumber *x) {
        return [ar2 map:^(NSNumber* y) {
            return @[x, y];
        }];
    }];
		
	NSArray *expected = @[@[@1, @1], @[@1, @2], @[@2, @1], @[@2, @2]];
	
	STAssertEqualObjects(newArray, expected, nil);
}

- (void)testTakeWhile {
    NSArray* array = @[@1, @2, @3, @4, @5, @6, @7, @8, @9, @10];
    
    NSArray* newArray = [array takeWhile:^BOOL(NSNumber* n) { return n.integerValue < 5; }];
    
    NSArray* expected = @[@1, @2, @3, @4];
    
    STAssertEqualObjects(newArray, expected, nil);
}

- (void)dropWhile {
    NSArray* array = @[@1, @2, @3, @4, @5, @6, @7, @8, @9, @10];
    
    NSArray* newArray = [array dropWhile:^BOOL(NSNumber* n) { return n.integerValue < 5; }];
    
    NSArray* expected = @[@5, @6, @7, @8, @9, @10];
    
    STAssertEqualObjects(newArray, expected, nil);
}

- (void)testMapOnSelf {
	NSMutableArray *array = [NSMutableArray arrayWithObjects:@"A", @"B", @"C", nil];
	
	NSMutableArray *result = [array mapOnSelf:^(NSString * e){ return [e stringByAppendingString:@"Y"]; }];

	STAssertTrue(array == result, @"[array mapOnSelf:] shoulud return itself");
	
	NSArray *expected = [NSArray arrayWithObjects:@"AY", @"BY", @"CY", nil];

	STAssertEqualObjects(array, expected, nil);
	
}

- (void)testFilter {
	NSArray *array = [NSArray arrayWithObjects:@"AA", @"AB", @"C", nil];
	
	NSArray *filteredArray = [array filter:^(NSString* e){ return [e hasPrefix:@"A"]; }];
	
	NSArray *expected = [NSArray arrayWithObjects:@"AA", @"AB", nil];
	
	STAssertEqualObjects(filteredArray, expected, nil);
}

- (void)testFilterNot {
	NSArray *array = [NSArray arrayWithObjects:@"AA", @"AB", @"C", nil];
	
	NSArray *filteredArray = [array filterNot:^(NSString* e){ return [e hasPrefix:@"A"]; }];
	
	NSArray *expected = [NSArray arrayWithObjects:@"C", nil];
	
	STAssertEqualObjects(filteredArray, expected, nil);
}

- (void)testFoldLeft {
	NSArray* array = [NSArray arrayWithObjects:[self number:1], [self number:2], [self number:3], nil];
	
	NSNumber* result = [array foldLeft:[self number:0] of:^(NSNumber* acc, NSNumber* e) {
		return [NSNumber numberWithInteger:[acc integerValue] + [e integerValue]];
	}];
	
	STAssertEqualObjects(result, [NSNumber numberWithInteger:6], nil);
}

- (void)testFoldRight {
	NSArray* array = [NSArray arrayWithObjects:[self number:1], [self number:2], [self number:3], nil];
	
	NSNumber* result = [array foldRight:[self number:0] of:^(NSNumber* e, NSNumber* acc) {
		NSLog(@" e = %@, acc = %@", e, acc);
		return [NSNumber numberWithInteger:[acc integerValue] - [e integerValue]];
	}];
	
	STAssertEqualObjects(result, [NSNumber numberWithInteger:-6], nil);
}

- (void)testJoin {
	NSArray* array = [NSArray arrayWithObjects:[NSNumber numberWithInt:100], [NSNumber numberWithInt:200], nil];
	NSString* result = [array join:@", "];
	STAssertEqualObjects(result, @"100, 200", nil);
}

- (void)testToSet {
    NSArray* array = @[@1, @2, @3, @4, @5];
    STAssertEqualObjects([array toSet], [NSSet setWithArray:array], nil);
    
    array = @[@1, @3, @1, @1, @3, @3];
    
    STAssertEqualObjects([array toSet], [NSSet setWithArray:(@[@1, @3])], nil);
}

- (void)testSumOfInt {
    NSArray* array = @[@1, @2, @3];
    NSObject* result = [array sum:[Monoids intMonoid]];
    
    STAssertEqualObjects(result, @6, nil);
    
    array = @[];
    
    result = [array sum:[Monoids intMonoid]];
    
    STAssertEqualObjects(result, @0, nil);
}

- (void)testSumOfDouble {
    NSArray* array = @[@1.5, @2.5, @3.5];
    NSObject* result = [array sum:[Monoids doubleMonoid]];
    
    STAssertEqualObjects(result, @7.5, nil);
    
    array = @[];
    
    result = [array sum:[Monoids doubleMonoid]];
    
    STAssertEqualObjects(result, @0.0, nil);
}

- (void)testSumOfString {
    NSArray* array = @[@"A", @"B", @"C"];
    NSObject* result = [array sum:[Monoids stringMonoid]];
    
    STAssertEqualObjects(result, @"ABC", nil);
    
    array = @[@"C", @"B", @"A"];
    
    result = [array sum:[Monoids stringMonoid]];
    
    STAssertEqualObjects(result, @"CBA", nil);
    
    array = @[];
    
    result = [array sum:[Monoids stringMonoid]];
    
    STAssertEqualObjects(result, @"", nil);
}


	
@end
