//
//  NSSetHigherOrderFunctionsTestCase.m
//  SubjectiveKit
//
//  Created by Kota Mizushima on 2012/10/19.
//
//

#import "NSSetHigherOrderFunctionsTestCase.h"
#import "NSSet+HigherOrderFunctions.h"
#import "Monoids.h"

@implementation NSSetHigherOrderFunctionsTestCase

- (NSNumber*)number:(NSInteger)value {
    return [NSNumber numberWithInteger:value];
}

- (void)testMap {
	NSSet*set = [NSSet setWithArray:@[@1, @2, @3, @4, @5]];
	
	NSSet *newSet = [set map:^(NSNumber* e){ return [self number:e.integerValue * 2]; }];
	
	NSSet *expected = [NSSet setWithArray:@[@2, @4, @6, @8, @10]];

    STAssertEqualObjects(newSet, expected, nil);
}

- (void)testFilter {
	NSSet *set = [NSSet setWithArray:@[@1, @2, @3, @4, @5]];
	
    NSSet *filteredSet = [set filter:^BOOL(NSNumber* e) { return e.integerValue < 4; }];
	
	NSSet *expected = [NSSet setWithArray:@[@1, @2, @3]];
	
	STAssertEqualObjects(filteredSet, expected, nil);
}

- (void)testFilterNot {
	NSSet *set = [NSSet setWithArray:@[@1, @2, @3, @4, @5]];
	
    NSSet *filteredSet = [set filter:^BOOL(NSNumber* e) { return e.integerValue >= 4; }];
	
	NSArray *expected = [NSSet setWithArray:@[@4, @5]];
	
	STAssertEqualObjects(filteredSet, expected, nil);
}

- (void)testFold {
    NSSet* set = [NSSet setWithArray:@[@1, @2, @3, @4, @5]];
    
	NSNumber* result = [set fold:[self number:0] of:^(NSNumber* acc, NSNumber* e) {
		return [NSNumber numberWithInteger:acc.intValue + e.intValue];
	}];
	
	STAssertEqualObjects(result, @15, nil);
}

- (void)testSumOfInt {
    NSSet* set = [NSSet setWithArray:@[@1, @2, @3]];
    NSObject* result = [set sum:[Monoids intMonoid]];
    
    STAssertEqualObjects(result, @6, nil);
    
    set = [NSSet setWithArray:@[]];
    
    result = [set sum:[Monoids intMonoid]];
    
    STAssertEqualObjects(result, @0, nil);
}

- (void)testSumOfDouble {
    NSSet* set = [NSSet setWithArray:@[@1.5, @2.5, @3.5]];
    NSObject* result = [set sum:[Monoids doubleMonoid]];
    
    STAssertEqualObjects(result, @7.5, nil);
    
    set = [NSSet setWithArray:@[]];
    
    result = [set sum:[Monoids doubleMonoid]];
    
    STAssertEqualObjects(result, @0.0, nil);
}

@end
